﻿var _ = require('underscore');
var fs = require('fs');

var mainTemplatePath = '';
var encoding = 'utf-8';

function hasValidMainTemplate( callback ) {
	
	fs.exists( mainTemplatePath, function( exists ) {
		return callback( exists );
	} );
	
}

module.exports.setMainTemplatePath = function( path ) {
	mainTemplatePath = path;
};

module.exports.setEncoding = function( e ) {
	encoding = e;
};

var  async = require('async');

module.exports.setupApp = function( app, views_path ) {
	
	app.engine( 'use', function( filePath, options, callback ) {
		
		async.series([
			function( callback ) {
				
				if ( options.page === undefined ) options.page = {};
				if ( options.page.include === undefined ) options.page.include = {};
				if ( options._includes === undefined ) options._includes = {};
				options.page.path = filePath;
				
				for ( var includeid in options._includes ) {
					
					var includeFileContent = fs.readFileSync( options._includes[ includeid ], encoding );
					var compiledInclude = _.template( includeFileContent );
					options.page.include[ includeid ] = compiledInclude( options );
					
				}
				
				var pageData = fs.readFileSync( filePath, encoding );
				var compiledPage = _.template( pageData );
				options.page.content = compiledPage( options );
				
				hasValidMainTemplate( function( exists ) {
					
					if (exists) {
					
						var templateData = fs.readFileSync( mainTemplatePath, encoding );
						var compiledTemplate = _.template( templateData );
						var rendered = compiledTemplate( options );
						
						callback( null, rendered );
						
					} else {
						
						var compiled = _.template( pageData );
						var rendered = compiled( options );
						
						callback( null, rendered );
						
					}
					
				} );
				
				return;
				
			}
		], function(err, datas) {
			callback(err, datas[0]);
		});
		
	} );
	
	app.set( 'views', views_path );
	app.set( 'view engine', 'use' );
	
}